@@ -6,7 +6,7 @@ You can use `complete_receive.py` to receive the message sent.
 
 """
 
-from kombu import Connection, Producer, Exchange, Queue
+from kombu import Connection, Exchange, Producer, Queue
 
 #: By default messages sent to exchanges are persistent (delivery_mode=2),
 #: and queues and exchanges are durable.
@@ -25,7 +25,10 @@ with Connection('amqp://guest:guest@localhost:5672//') as connection:
     #: Publish the message using the json serializer (which is the default),
     #: and zlib compression.  The kombu consumer will automatically detect
     #: encoding, serialization and compression used and decode accordingly.
-    producer.publish({'hello': 'world'},
-                     exchange=exchange,
-                     routing_key='kombu_demo',
-                     serializer='json', compression='zlib')
+    producer.publish(
+        {'hello': 'world'},
+        exchange=exchange,
+        routing_key='kombu_demo',
+        serializer='json',
+        compression='zlib',
+    )
