@@ -1,5 +1,4 @@
-from kombu import Connection  # noqa
-
+from kombu import Connection
 
 with Connection('amqp://guest:guest@localhost:5672//') as conn:
     simple_queue = conn.SimpleQueue('simple_queue')
