@@ -1,16 +1,14 @@
 """Carrot compatibility interface.
 
-See https://pypi.python.org/pypi/carrot for documentation.
+See https://pypi.org/project/carrot/ for documentation.
 """
-from __future__ import absolute_import, unicode_literals
 
 from itertools import count
 
 from . import messaging
 from .entity import Exchange, Queue
-from .five import items
 
-__all__ = ['Publisher', 'Consumer']
+__all__ = ('Publisher', 'Consumer')
 
 # XXX compat attribute
 entry_to_queue = Queue.from_dict
@@ -55,13 +53,13 @@ class Publisher(messaging.Producer):
                                      routing_key=self.routing_key,
                                      auto_delete=self.auto_delete,
                                      durable=self.durable)
-        super(Publisher, self).__init__(connection, self.exchange, **kwargs)
+        super().__init__(connection, self.exchange, **kwargs)
 
     def send(self, *args, **kwargs):
         return self.publish(*args, **kwargs)
 
     def close(self):
-        super(Publisher, self).close()
+        super().close()
         self._closed = True
 
     def __enter__(self):
@@ -115,11 +113,11 @@ class Consumer(messaging.Consumer):
                       durable=self.durable,
                       exclusive=self.exclusive,
                       auto_delete=self.auto_delete)
-        super(Consumer, self).__init__(self.backend, queue, **kwargs)
+        super().__init__(self.backend, queue, **kwargs)
 
     def revive(self, channel):
         self.backend = channel
-        super(Consumer, self).revive(channel)
+        super().revive(channel)
 
     def close(self):
         self.cancel()
@@ -185,10 +183,10 @@ class ConsumerSet(messaging.Consumer):
             for consumer in consumers:
                 queues.extend(consumer.queues)
         if from_dict:
-            for queue_name, queue_options in items(from_dict):
+            for queue_name, queue_options in from_dict.items():
                 queues.append(Queue.from_dict(queue_name, **queue_options))
 
-        super(ConsumerSet, self).__init__(self.backend, queues, **kwargs)
+        super().__init__(self.backend, queues, **kwargs)
 
     def iterconsume(self, limit=None, no_ack=False):
         return _iterconsume(self.connection, self, no_ack, limit)
@@ -205,7 +203,7 @@ class ConsumerSet(messaging.Consumer):
 
     def revive(self, channel):
         self.backend = channel
-        super(ConsumerSet, self).revive(channel)
+        super().revive(channel)
 
     def close(self):
         self.cancel()
