@@ -1,26 +1,22 @@
 """Generic process mailbox."""
-from __future__ import absolute_import, unicode_literals
 
 import socket
 import warnings
-
 from collections import defaultdict, deque
 from contextlib import contextmanager
 from copy import copy
 from itertools import count
-from threading import local
 from time import time
 
-from . import Exchange, Queue, Consumer, Producer
+from . import Consumer, Exchange, Producer, Queue
 from .clocks import LamportClock
 from .common import maybe_declare, oid_from
 from .exceptions import InconsistencyError
-from .five import range, string_t
 from .log import get_logger
+from .matcher import match
 from .utils.functional import maybe_evaluate, reprcall
 from .utils.objects import cached_property
 from .utils.uuid import uuid
-from .matcher import match
 
 REPLY_QUEUE_EXPIRES = 10
 
@@ -37,7 +33,7 @@ logger = get_logger(__name__)
 debug, error = logger.debug, logger.error
 
 
-class Node(object):
+class Node:
     """Mailbox node."""
 
     #: hostname of the node.
@@ -151,7 +147,7 @@ class Node(object):
                                     serializer=self.mailbox.serializer)
 
 
-class Mailbox(object):
+class Mailbox:
     """Process Mailbox."""
 
     node_cls = Node
@@ -190,7 +186,6 @@ class Mailbox(object):
         self.clock = LamportClock() if clock is None else clock
         self.exchange = self._get_exchange(self.namespace, self.type)
         self.reply_exchange = self._get_reply_exchange(self.namespace)
-        self._tls = local()
         self.unclaimed = defaultdict(deque)
         self.accept = self.accept if accept is None else accept
         self.serializer = self.serializer if serializer is None else serializer
@@ -236,7 +231,7 @@ class Mailbox(object):
     def get_reply_queue(self):
         oid = self.oid
         return Queue(
-            '%s.%s' % (oid, self.reply_exchange.name),
+            f'{oid}.{self.reply_exchange.name}',
             exchange=self.reply_exchange,
             routing_key=oid,
             durable=False,
@@ -251,7 +246,7 @@ class Mailbox(object):
 
     def get_queue(self, hostname):
         return Queue(
-            '%s.%s.pidbox' % (hostname, self.namespace),
+            f'{hostname}.{self.namespace}.pidbox',
             exchange=self.exchange,
             durable=False,
             auto_delete=True,
@@ -299,7 +294,7 @@ class Mailbox(object):
         chan = channel or self.connection.default_channel
         exchange = self.exchange
         if reply_ticket:
-            maybe_declare(self.reply_queue(channel))
+            maybe_declare(self.reply_queue(chan))
             message.update(ticket=reply_ticket,
                            reply_to={'exchange': self.reply_exchange.name,
                                      'routing_key': self.oid})
@@ -319,10 +314,10 @@ class Mailbox(object):
         if destination is not None and \
                 not isinstance(destination, (list, tuple)):
             raise ValueError(
-                'destination must be a list/tuple not {0}'.format(
+                'destination must be a list/tuple not {}'.format(
                     type(destination)))
-        if (pattern is not None and not isinstance(pattern, string_t) and
-                matcher is not None and not isinstance(matcher, string_t)):
+        if (pattern is not None and not isinstance(pattern, str) and
+                matcher is not None and not isinstance(matcher, str)):
             raise ValueError(
                 'pattern and matcher must be '
                 'strings not {}, {}'.format(type(pattern), type(matcher))
@@ -358,7 +353,7 @@ class Mailbox(object):
             accept = self.accept
         chan = channel or self.connection.default_channel
         queue = self.reply_queue
-        consumer = Consumer(channel, [queue], accept=accept, no_ack=True)
+        consumer = Consumer(chan, [queue], accept=accept, no_ack=True)
         responses = []
         unclaimed = self.unclaimed
         adjust_clock = self.clock.adjust
@@ -407,13 +402,9 @@ class Mailbox(object):
                         durable=False,
                         delivery_mode='transient')
 
-    @cached_property
+    @property
     def oid(self):
-        try:
-            return self._tls.OID
-        except AttributeError:
-            oid = self._tls.OID = oid_from(self)
-            return oid
+        return oid_from(self)
 
     @cached_property
     def producer_pool(self):
