@@ -1,12 +1,9 @@
 """Public resource pools."""
-from __future__ import absolute_import, unicode_literals
 
 import os
-
 from itertools import chain
 
 from .connection import Resource
-from .five import range, values
 from .messaging import Producer
 from .utils.collections import EqualityDict
 from .utils.compat import register_after_fork
@@ -33,7 +30,7 @@ class ProducerPool(Resource):
     def __init__(self, connections, *args, **kwargs):
         self.connections = connections
         self.Producer = kwargs.pop('Producer', None) or self.Producer
-        super(ProducerPool, self).__init__(*args, **kwargs)
+        super().__init__(*args, **kwargs)
 
     def _acquire_connection(self):
         return self.connections.acquire(block=True)
@@ -73,7 +70,7 @@ class ProducerPool(Resource):
         if resource.__connection__:
             resource.__connection__.release()
         resource.channel = None
-        super(ProducerPool, self).release(resource)
+        super().release(resource)
 
 
 class PoolGroup(EqualityDict):
@@ -107,7 +104,9 @@ class Connections(PoolGroup):
 
     def create(self, connection, limit):
         return connection.Pool(limit=limit)
-connections = register_group(Connections(limit=use_global_limit))  # noqa: E305
+
+
+connections = register_group(Connections(limit=use_global_limit))
 
 
 class Producers(PoolGroup):
@@ -115,11 +114,13 @@ class Producers(PoolGroup):
 
     def create(self, connection, limit):
         return ProducerPool(connections[connection], limit=limit)
-producers = register_group(Producers(limit=use_global_limit))  # noqa: E305
+
+
+producers = register_group(Producers(limit=use_global_limit))
 
 
 def _all_pools():
-    return chain(*[(values(g) if g else iter([])) for g in _groups])
+    return chain(*((g.values() if g else iter([])) for g in _groups))
 
 
 def get_limit():
