@@ -1,22 +1,31 @@
+"""SoftLayer Message Queue transport module for kombu.
+
+Features
+========
+* Type: Virtual
+* Supports Direct: Yes
+* Supports Topic: Yes
+* Supports Fanout: No
+* Supports Priority: No
+* Supports TTL: No
+
+Connection String
+=================
+ *Unreviewed*
+
+Transport Options
+=================
+ *Unreviewed*
 """
-kombu.transport.SLMQ
-====================
-
-SoftLayer Message Queue transport.
-
-"""
-from __future__ import absolute_import
-
-import socket
-import string
-
-from anyjson import loads, dumps
 
 import os
+import socket
+import string
+from queue import Empty
 
-from kombu.five import Empty, text_t
-from kombu.utils import cached_property  # , uuid
 from kombu.utils.encoding import bytes_to_str, safe_str
+from kombu.utils.json import dumps, loads
+from kombu.utils.objects import cached_property
 
 from . import virtual
 
@@ -24,14 +33,17 @@ try:
     from softlayer_messaging import get_client
     from softlayer_messaging.errors import ResponseError
 except ImportError:  # pragma: no cover
-    get_client = ResponseError = None  # noqa
+    get_client = ResponseError = None
 
 # dots are replaced by dash, all other punctuation replaced by underscore.
-CHARS_REPLACE_TABLE = dict(
-    (ord(c), 0x5f) for c in string.punctuation if c not in '_')
+CHARS_REPLACE_TABLE = {
+    ord(c): 0x5f for c in string.punctuation if c not in '_'
+}
 
 
 class Channel(virtual.Channel):
+    """SLMQ Channel."""
+
     default_visibility_timeout = 1800  # 30 minutes.
     domain_format = 'kombu%(vhost)s'
     _slmq = None
@@ -43,7 +55,7 @@ class Channel(virtual.Channel):
             raise ImportError(
                 'SLMQ transport requires the softlayer_messaging library',
             )
-        super(Channel, self).__init__(*args, **kwargs)
+        super().__init__(*args, **kwargs)
         queues = self.slmq.queues()
         for queue in queues:
             self._queue_cache[queue] = queue
@@ -51,21 +63,21 @@ class Channel(virtual.Channel):
     def basic_consume(self, queue, no_ack, *args, **kwargs):
         if no_ack:
             self._noack_queues.add(queue)
-        return super(Channel, self).basic_consume(queue, no_ack,
-                                                  *args, **kwargs)
+        return super().basic_consume(queue, no_ack,
+                                     *args, **kwargs)
 
     def basic_cancel(self, consumer_tag):
         if consumer_tag in self._consumers:
             queue = self._tag_to_queue[consumer_tag]
             self._noack_queues.discard(queue)
-        return super(Channel, self).basic_cancel(consumer_tag)
+        return super().basic_cancel(consumer_tag)
 
     def entity_name(self, name, table=CHARS_REPLACE_TABLE):
         """Format AMQP queue name into a valid SLQS queue name."""
-        return text_t(safe_str(name)).translate(table)
+        return str(safe_str(name)).translate(table)
 
     def _new_queue(self, queue, **kwargs):
-        """Ensures a queue exists in SLQS."""
+        """Ensure a queue exists in SLQS."""
         queue = self.entity_name(self.queue_name_prefix + queue)
         try:
             return self._queue_cache[queue]
@@ -78,12 +90,12 @@ class Channel(virtual.Channel):
             q = self._queue_cache[queue] = self.slmq.queue(queue)
             return q
 
-    def _delete(self, queue, *args):
-        """delete queue by name."""
+    def _delete(self, queue, *args, **kwargs):
+        """Delete queue by name."""
         queue_name = self.entity_name(queue)
         self._queue_cache.pop(queue_name, None)
         self.slmq.queue(queue_name).delete(force=True)
-        super(Channel, self)._delete(queue_name)
+        super()._delete(queue_name)
 
     def _put(self, queue, message, **kwargs):
         """Put message onto queue."""
@@ -113,7 +125,7 @@ class Channel(virtual.Channel):
             pass
         else:
             self.delete_message(queue, delivery_info['slmq_message_id'])
-        super(Channel, self).basic_ack(delivery_tag)
+        super().basic_ack(delivery_tag)
 
     def _size(self, queue):
         """Return the number of messages in a queue."""
@@ -123,12 +135,12 @@ class Channel(virtual.Channel):
         """Delete all current messages in a queue."""
         q = self._new_queue(queue)
         n = 0
-        l = q.pop(10)
-        while l['items']:
-            for m in l['items']:
+        results = q.pop(10)
+        while results['items']:
+            for m in results['items']:
                 self.delete_message(queue, m['id'])
                 n += 1
-            l = q.pop(10)
+            results = q.pop(10)
         return n
 
     def delete_message(self, queue, message_id):
@@ -145,13 +157,12 @@ class Channel(virtual.Channel):
             host = os.environ.get('SLMQ_HOST', conninfo.hostname)
             port = os.environ.get('SLMQ_PORT', conninfo.port)
             secure = bool(os.environ.get(
-                'SLMQ_SECURE', self.transport_options.get('secure')) or True)
-            if secure:
-                endpoint = "https://%s" % host
-            else:
-                endpoint = "http://%s" % host
-            if port:
-                endpoint = "%s:%s" % (endpoint, port)
+                'SLMQ_SECURE', self.transport_options.get('secure')) or True,
+            )
+            endpoint = '{}://{}{}'.format(
+                'https' if secure else 'http', host,
+                f':{port}' if port else '',
+            )
 
             self._slmq = get_client(account, endpoint=endpoint)
             self._slmq.authenticate(user, api_key)
@@ -176,6 +187,8 @@ class Channel(virtual.Channel):
 
 
 class Transport(virtual.Transport):
+    """SLMQ Transport."""
+
     Channel = Channel
 
     polling_interval = 1
