@@ -1,21 +1,106 @@
-"""File-system Transport.
+"""File-system Transport module for kombu.
 
-Transport using the file-system as the message store.
+Transport using the file-system as the message store. Messages written to the
+queue are stored in `data_folder_in` directory and
+messages read from the queue are read from `data_folder_out` directory. Both
+directories must be created manually. Simple example:
+
+* Producer:
+
+.. code-block:: python
+
+    import kombu
+
+    conn = kombu.Connection(
+        'filesystem://', transport_options={
+            'data_folder_in': 'data_in', 'data_folder_out': 'data_out'
+        }
+    )
+    conn.connect()
+
+    test_queue = kombu.Queue('test', routing_key='test')
+
+    with conn as conn:
+        with conn.default_channel as channel:
+            producer = kombu.Producer(channel)
+            producer.publish(
+                        {'hello': 'world'},
+                        retry=True,
+                        exchange=test_queue.exchange,
+                        routing_key=test_queue.routing_key,
+                        declare=[test_queue],
+                        serializer='pickle'
+            )
+
+* Consumer:
+
+.. code-block:: python
+
+    import kombu
+
+    conn = kombu.Connection(
+        'filesystem://', transport_options={
+            'data_folder_in': 'data_out', 'data_folder_out': 'data_in'
+        }
+    )
+    conn.connect()
+
+    def callback(body, message):
+        print(body, message)
+        message.ack()
+
+    test_queue = kombu.Queue('test', routing_key='test')
+
+    with conn as conn:
+        with conn.default_channel as channel:
+            consumer = kombu.Consumer(
+                conn, [test_queue], accept=['pickle']
+            )
+            consumer.register_callback(callback)
+            with consumer:
+                conn.drain_events(timeout=1)
+
+Features
+========
+* Type: Virtual
+* Supports Direct: Yes
+* Supports Topic: Yes
+* Supports Fanout: No
+* Supports Priority: No
+* Supports TTL: No
+
+Connection String
+=================
+Connection string is in the following format:
+
+.. code-block::
+
+    filesystem://
+
+Transport Options
+=================
+* ``data_folder_in`` - directory where are messages stored when written
+  to queue.
+* ``data_folder_out`` - directory from which are messages read when read from
+  queue.
+* ``store_processed`` - if set to True, all processed messages are backed up to
+  ``processed_folder``.
+* ``processed_folder`` - directory where are backed up processed files.
 """
-from __future__ import absolute_import, unicode_literals
 
 import os
 import shutil
-import uuid
 import tempfile
+import uuid
+from queue import Empty
+from time import monotonic
 
-from . import virtual
 from kombu.exceptions import ChannelError
-from kombu.five import Empty, monotonic
 from kombu.utils.encoding import bytes_to_str, str_to_bytes
-from kombu.utils.json import loads, dumps
+from kombu.utils.json import dumps, loads
 from kombu.utils.objects import cached_property
 
+from . import virtual
 
 VERSION = (1, 0, 0)
 __version__ = '.'.join(map(str, VERSION))
@@ -23,14 +108,14 @@ __version__ = '.'.join(map(str, VERSION))
 # needs win32all to work on Windows
 if os.name == 'nt':
 
+    import pywintypes
     import win32con
     import win32file
-    import pywintypes
 
     LOCK_EX = win32con.LOCKFILE_EXCLUSIVE_LOCK
     # 0 is the default
-    LOCK_SH = 0                                     # noqa
-    LOCK_NB = win32con.LOCKFILE_FAIL_IMMEDIATELY    # noqa
+    LOCK_SH = 0
+    LOCK_NB = win32con.LOCKFILE_FAIL_IMMEDIATELY
     __overlapped = pywintypes.OVERLAPPED()
 
     def lock(file, flags):
@@ -46,13 +131,13 @@ if os.name == 'nt':
 elif os.name == 'posix':
 
     import fcntl
-    from fcntl import LOCK_EX, LOCK_SH, LOCK_NB     # noqa
+    from fcntl import LOCK_EX, LOCK_NB, LOCK_SH  # noqa
 
-    def lock(file, flags):  # noqa
+    def lock(file, flags):
         """Create file lock."""
         fcntl.flock(file.fileno(), flags)
 
-    def unlock(file):       # noqa
+    def unlock(file):
         """Remove file lock."""
         fcntl.flock(file.fileno(), fcntl.LOCK_UN)
 else:
@@ -65,17 +150,17 @@ class Channel(virtual.Channel):
 
     def _put(self, queue, payload, **kwargs):
         """Put `message` onto `queue`."""
-        filename = '%s_%s.%s.msg' % (int(round(monotonic() * 1000)),
-                                     uuid.uuid4(), queue)
+        filename = '{}_{}.{}.msg'.format(int(round(monotonic() * 1000)),
+                                         uuid.uuid4(), queue)
         filename = os.path.join(self.data_folder_out, filename)
 
         try:
             f = open(filename, 'wb')
             lock(f, LOCK_EX)
             f.write(str_to_bytes(dumps(payload)))
-        except (IOError, OSError):
+        except OSError:
             raise ChannelError(
-                'Cannot add file {0!r} to directory'.format(filename))
+                f'Cannot add file {filename!r} to directory')
         finally:
             unlock(f)
             f.close()
@@ -101,7 +186,7 @@ class Channel(virtual.Channel):
                 # move the file to the tmp/processed folder
                 shutil.move(os.path.join(self.data_folder_in, filename),
                             processed_folder)
-            except IOError:
+            except OSError:
                 pass  # file could be locked, or removed in meantime so ignore
 
             filename = os.path.join(processed_folder, filename)
@@ -111,9 +196,9 @@ class Channel(virtual.Channel):
                 f.close()
                 if not self.store_processed:
                     os.remove(filename)
-            except (IOError, OSError):
+            except OSError:
                 raise ChannelError(
-                    'Cannot read file {0!r} from queue.'.format(filename))
+                    f'Cannot read file {filename!r} from queue.')
 
             return loads(bytes_to_str(payload))
 
@@ -148,7 +233,7 @@ class Channel(virtual.Channel):
         """Return the number of messages in `queue` as an :class:`int`."""
         count = 0
 
-        queue_find = '.{0}.msg'.format(queue)
+        queue_find = f'.{queue}.msg'
         folder = os.listdir(self.data_folder_in)
         while len(folder) > 0:
             filename = folder.pop()
@@ -186,10 +271,15 @@ class Transport(virtual.Transport):
     """Filesystem Transport."""
 
     Channel = Channel
-
+    # filesystem backend state is global.
+    global_state = virtual.BrokerState()
     default_port = 0
     driver_type = 'filesystem'
     driver_name = 'filesystem'
 
+    def __init__(self, client, **kwargs):
+        super().__init__(client, **kwargs)
+        self.state = self.global_state
+
     def driver_version(self):
         return 'N/A'
