@@ -2,7 +2,6 @@
 
 .. _`librabbitmq`: https://pypi.org/project/librabbitmq/
 """
-from __future__ import absolute_import, unicode_literals
 
 import os
 import socket
@@ -11,7 +10,6 @@ import warnings
 import librabbitmq as amqp
 from librabbitmq import ChannelError, ConnectionError
 
-from kombu.five import items, values
 from kombu.utils.amq_manager import get_manager
 from kombu.utils.text import version_string_as_tuple
 
@@ -34,7 +32,7 @@ class Message(base.Message):
     """AMQP Message (librabbitmq)."""
 
     def __init__(self, channel, props, info, body):
-        super(Message, self).__init__(
+        super().__init__(
             channel=channel,
             body=body,
             delivery_info=info,
@@ -67,7 +65,7 @@ class Channel(amqp.Channel, base.StdChannel):
 
     def prepare_queue_arguments(self, arguments, **kwargs):
         arguments = to_rabbitmq_queue_arguments(arguments, **kwargs)
-        return {k.encode('utf8'): v for k, v in items(arguments)}
+        return {k.encode('utf8'): v for k, v in arguments.items()}
 
 
 class Connection(amqp.Connection):
@@ -119,7 +117,7 @@ class Transport(base.Transport):
     def establish_connection(self):
         """Establish connection to the AMQP broker."""
         conninfo = self.client
-        for name, default_value in items(self.default_connection_params):
+        for name, default_value in self.default_connection_params.items():
             if not getattr(conninfo, name, None):
                 setattr(conninfo, name, default_value)
         if conninfo.ssl:
@@ -146,7 +144,7 @@ class Transport(base.Transport):
 
     def _collect(self, connection):
         if connection is not None:
-            for channel in values(connection.channels):
+            for channel in connection.channels.values():
                 channel.connection = None
             try:
                 os.close(connection.fileno())
@@ -186,5 +184,5 @@ class Transport(base.Transport):
             'port': (self.default_ssl_port if self.client.ssl
                      else self.default_port),
             'hostname': 'localhost',
-            'login_method': 'AMQPLAIN',
+            'login_method': 'PLAIN',
         }
