@@ -2,34 +2,31 @@
 
 Emulates the AMQ API for non-AMQ transports.
 """
-from __future__ import absolute_import, print_function, unicode_literals
 
 import base64
 import socket
 import sys
 import warnings
-
 from array import array
 from collections import OrderedDict, defaultdict, namedtuple
 from itertools import count
 from multiprocessing.util import Finalize
-from time import sleep
+from queue import Empty
+from time import monotonic, sleep
 
 from amqp.protocol import queue_declare_ok_t
 
-from kombu.exceptions import ResourceError, ChannelError
-from kombu.five import Empty, items, monotonic
+from kombu.exceptions import ChannelError, ResourceError
 from kombu.log import get_logger
-from kombu.utils.encoding import str_to_bytes, bytes_to_str
+from kombu.transport import base
 from kombu.utils.div import emergency_dump_state
+from kombu.utils.encoding import bytes_to_str, str_to_bytes
 from kombu.utils.scheduling import FairCycle
 from kombu.utils.uuid import uuid
 
-from kombu.transport import base
-
 from .exchange import STANDARD_EXCHANGE_TYPES
 
-ARRAY_TYPE_H = 'H' if sys.version_info[0] == 3 else b'H'
+ARRAY_TYPE_H = 'H'
 
 UNDELIVERABLE_FMT = """\
 Message could not be delivered: No queues bound to exchange {exchange!r} \
@@ -61,7 +58,7 @@ queue_binding_t = namedtuple('queue_binding_t', (
 ))
 
 
-class Base64(object):
+class Base64:
     """Base64 codec."""
 
     def encode(self, s):
@@ -79,7 +76,7 @@ class UndeliverableWarning(UserWarning):
     """The message could not be delivered to a queue."""
 
 
-class BrokerState(object):
+class BrokerState:
     """Broker state holds exchanges, queues and bindings."""
 
     #: Mapping of exchange name to
@@ -98,6 +95,7 @@ class BrokerState(object):
 
     #: The queue index is used to access directly (constant time)
     #: all the bindings of a certain queue.  It has the following structure::
+    #:
     #:     {
     #:         queue: {
     #:             (queue, exchange, routing_key),
@@ -149,7 +147,7 @@ class BrokerState(object):
         )
 
 
-class QoS(object):
+class QoS:
     """Quality of Service guarantees.
 
     Only supports `prefetch_count` at this point.
@@ -303,7 +301,6 @@ class QoS(object):
             This is implementation optional, and currently only
             used by the Redis transport.
         """
-        pass
 
 
 class Message(base.Message):
@@ -315,7 +312,7 @@ class Message(base.Message):
         body = payload.get('body')
         if body:
             body = channel.decode_body(body, properties.get('body_encoding'))
-        super(Message, self).__init__(
+        super().__init__(
             body=body,
             channel=channel,
             delivery_tag=properties['delivery_tag'],
@@ -343,7 +340,7 @@ class Message(base.Message):
         }
 
 
-class AbstractChannel(object):
+class AbstractChannel:
     """Abstract channel interface.
 
     This is an abstract class defining the channel methods
@@ -386,7 +383,6 @@ class AbstractChannel(object):
             Your transport can override this method if it needs
             to do something whenever a new queue is declared.
         """
-        pass
 
     def _has_queue(self, queue, **kwargs):
         """Verify that queue exists.
@@ -463,17 +459,10 @@ class Channel(AbstractChannel, base.StdChannel):
 
         # instantiate exchange types
         self.exchange_types = {
-            typ: cls(self) for typ, cls in items(self.exchange_types)
+            typ: cls(self) for typ, cls in self.exchange_types.items()
         }
 
-        try:
-            self.channel_id = self.connection._avail_channel_ids.pop()
-        except IndexError:
-            raise ResourceError(
-                'No free channel ids, current={0}, channel_max={1}'.format(
-                    len(self.connection.channels),
-                    self.connection.channel_max), (20, 10),
-            )
+        self.channel_id = self._get_free_channel_id()
 
         topts = self.connection.client.transport_options
         for opt_name in self.from_transport_options:
@@ -491,7 +480,7 @@ class Channel(AbstractChannel, base.StdChannel):
         if passive:
             if exchange not in self.state.exchanges:
                 raise ChannelError(
-                    'NOT_FOUND - no exchange {0!r} in vhost {1!r}'.format(
+                    'NOT_FOUND - no exchange {!r} in vhost {!r}'.format(
                         exchange, self.connection.client.virtual_host or '/'),
                     (50, 10), 'Channel.exchange_declare', '404',
                 )
@@ -523,7 +512,7 @@ class Channel(AbstractChannel, base.StdChannel):
         queue = queue or 'amq.gen-%s' % uuid()
         if passive and not self._has_queue(queue, **kwargs):
             raise ChannelError(
-                'NOT_FOUND - no queue {0!r} in vhost {1!r}'.format(
+                'NOT_FOUND - no queue {!r} in vhost {!r}'.format(
                     queue, self.connection.client.virtual_host or '/'),
                 (50, 10), 'Channel.queue_declare', '404',
             )
@@ -848,12 +837,28 @@ class Channel(AbstractChannel, base.StdChannel):
 
         return (self.max_priority - priority) if reverse else priority
 
+    def _get_free_channel_id(self):
+        # Cast to a set for fast lookups, and keep stored as an array
+        # for lower memory usage.
+        used_channel_ids = set(self.connection._used_channel_ids)
+
+        for channel_id in range(1, self.connection.channel_max + 1):
+            if channel_id not in used_channel_ids:
+                self.connection._used_channel_ids.append(channel_id)
+                return channel_id
+
+        raise ResourceError(
+            'No free channel ids, current={}, channel_max={}'.format(
+                len(self.connection.channels),
+                self.connection.channel_max), (20, 10),
+        )
+
 
 class Management(base.Management):
     """Base class for the AMQP management API."""
 
     def __init__(self, transport):
-        super(Management, self).__init__(transport)
+        super().__init__(transport)
         self.channel = transport.client.channel()
 
     def get_bindings(self):
@@ -875,9 +880,6 @@ class Transport(base.Transport):
     Cycle = FairCycle
     Management = Management
 
-    #: Global :class:`BrokerState` containing declared exchanges and bindings.
-    state = BrokerState()
-
     #: :class:`~kombu.utils.scheduling.FairCycle` instance
     #: used to fairly drain events from channels (set by constructor).
     cycle = None
@@ -905,6 +907,8 @@ class Transport(base.Transport):
 
     def __init__(self, client, **kwargs):
         self.client = client
+        # :class:`BrokerState` containing declared exchanges and bindings.
+        self.state = BrokerState()
         self.channels = []
         self._avail_channels = []
         self._callbacks = {}
@@ -912,9 +916,7 @@ class Transport(base.Transport):
         polling_interval = client.transport_options.get('polling_interval')
         if polling_interval is not None:
             self.polling_interval = polling_interval
-        self._avail_channel_ids = array(
-            ARRAY_TYPE_H, range(self.channel_max, 0, -1),
-        )
+        self._used_channel_ids = array(ARRAY_TYPE_H)
 
     def create_channel(self, connection):
         try:
@@ -926,7 +928,11 @@ class Transport(base.Transport):
 
     def close_channel(self, channel):
         try:
-            self._avail_channel_ids.append(channel.channel_id)
+            try:
+                self._used_channel_ids.remove(channel.channel_id)
+            except ValueError:
+                # channel id already removed
+                pass
             try:
                 self.channels.remove(channel)
             except ValueError:
@@ -972,7 +978,7 @@ class Transport(base.Transport):
     def _deliver(self, message, queue):
         if not queue:
             raise KeyError(
-                'Received message without destination queue: {0}'.format(
+                'Received message without destination queue: {}'.format(
                     message))
         try:
             callback = self._callbacks[queue]
@@ -993,7 +999,7 @@ class Transport(base.Transport):
     def on_message_ready(self, channel, message, queue):
         if not queue or queue not in self._callbacks:
             raise KeyError(
-                'Message for queue {0!r} without consumers: {1}'.format(
+                'Message for queue {!r} without consumers: {}'.format(
                     queue, message))
         self._callbacks[queue](message)
 
