@@ -1,13 +1,10 @@
-from unittest.mock import Mock, MagicMock
+from unittest.mock import MagicMock, Mock
 
-from kombu.asynchronous.aws.sqs.connection import (
-    AsyncSQSConnection
-)
 from kombu.asynchronous.aws.ext import boto3
+from kombu.asynchronous.aws.sqs.connection import AsyncSQSConnection
 from kombu.asynchronous.aws.sqs.message import AsyncMessage
 from kombu.asynchronous.aws.sqs.queue import AsyncQueue
 from kombu.utils.uuid import uuid
-
 from t.mocks import PromiseMock
 
 from ..case import AWSCase
@@ -90,7 +87,10 @@ class test_AsyncSQSConnection(AWSCase):
             callback=self.callback,
         )
         self.x.get_list.assert_called_with(
-            'ReceiveMessage', {'MaxNumberOfMessages': 4},
+            'ReceiveMessage', {
+                'MaxNumberOfMessages': 4,
+                'AttributeName.1': 'ApproximateReceiveCount'
+            },
             [('Message', AsyncMessage)],
             'http://aws.com', callback=self.callback,
             parent=queue,
@@ -109,6 +109,7 @@ class test_AsyncSQSConnection(AWSCase):
             'ReceiveMessage', {
                 'MaxNumberOfMessages': 4,
                 'VisibilityTimeout': 3666,
+                'AttributeName.1': 'ApproximateReceiveCount',
             },
             [('Message', AsyncMessage)],
             'http://aws.com', callback=self.callback,
@@ -128,6 +129,7 @@ class test_AsyncSQSConnection(AWSCase):
             'ReceiveMessage', {
                 'MaxNumberOfMessages': 4,
                 'WaitTimeSeconds': 303,
+                'AttributeName.1': 'ApproximateReceiveCount',
             },
             [('Message', AsyncMessage)],
             'http://aws.com', callback=self.callback,
