@@ -1,4 +1,4 @@
-from case import Mock
+from unittest.mock import Mock
 
 from kombu.asynchronous.aws import connect_sqs
 
