@@ -1,12 +1,14 @@
+from unittest.mock import Mock, call, patch
+
 import pytest
 
-from case import Mock, call, patch, skip
-
+import t.skip
 from kombu.asynchronous.http.curl import READ, WRITE, CurlClient
 
+pytest.importorskip('pycurl')
 
-@skip.if_pypy()
-@skip.unless_module('pycurl')
+
+@t.skip.if_pypy
 @pytest.mark.usefixtures('hub')
 class test_CurlClient:
 
@@ -92,8 +94,10 @@ class test_CurlClient:
             x._handle_socket(0xff3f, fd, x._multi, None, _pycurl)
 
     def test_set_timeout(self):
-        x = self.Client()
+        hub = Mock(name='hub')
+        x = self.Client(hub)
         x._set_timeout(100)
+        hub.call_later.assert_called_with(100, x._timeout_check)
 
     def test_timeout_check(self):
         with patch('kombu.asynchronous.http.curl.pycurl') as _pycurl:
