@@ -1,8 +1,9 @@
-import pytest
-
 from unittest.mock import Mock
 
+import pytest
+
 from kombu import Connection, Exchange, Queue
+from kombu.exceptions import ContentDisallowed
 
 
 class SimpleBase:
@@ -21,13 +22,10 @@ class SimpleBase:
     def setup(self):
         self.connection = Connection(transport='memory')
         self.connection.default_channel.exchange_declare('amq.direct')
-        self.q = self.Queue(None, no_ack=True)
 
     def teardown(self):
-        self.q.close()
         self.connection.close()
         self.connection = None
-        self.q = None
 
     def test_produce__consume(self):
         q = self.Queue('test_produce__consume', no_ack=True)
@@ -50,6 +48,38 @@ class SimpleBase:
         with pytest.raises(q.Empty):
             q.get(block=False)
 
+    def test_get_nowait_accept(self):
+        q = self.Queue('test_accept', serializer='pickle', accept=['json'])
+        q.put({'hello': 'SimpleSync'})
+        with pytest.raises(ContentDisallowed):
+            q.get_nowait().payload
+
+        q = self.Queue('test_accept1', serializer='json', accept=[])
+        q.put({'hello': 'SimpleSync'})
+        with pytest.raises(ContentDisallowed):
+            q.get_nowait().payload
+
+        q = self.Queue(
+            'test_accept2', serializer='pickle', accept=['json', 'pickle'])
+        q.put({'hello': 'SimpleSync'})
+        assert q.get_nowait().payload == {'hello': 'SimpleSync'}
+
+    def test_get_accept(self):
+        q = self.Queue('test_accept', serializer='pickle', accept=['json'])
+        q.put({'hello': 'SimpleSync'})
+        with pytest.raises(ContentDisallowed):
+            q.get().payload
+
+        q = self.Queue('test_accept1', serializer='pickle', accept=[])
+        q.put({'hello': 'SimpleSync'})
+        with pytest.raises(ContentDisallowed):
+            q.get().payload
+
+        q = self.Queue(
+            'test_accept2', serializer='pickle', accept=['json', 'pickle'])
+        q.put({'hello': 'SimpleSync'})
+        assert q.get().payload == {'hello': 'SimpleSync'}
+
     def test_clear(self):
         q = self.Queue('test_clear', no_ack=True)
 
