@@ -4,23 +4,23 @@ NOTE: The SQSQueueMock and SQSConnectionMock classes originally come from
 http://github.com/pcsforeducation/sqs-mock-python. They have been patched
 slightly.
 """
-
-
+import base64
 import os
-import pytest
 import random
 import string
+from datetime import datetime, timedelta
 from queue import Empty
-
 from unittest.mock import Mock, patch
 
-from kombu import messaging
-from kombu import Connection, Exchange, Queue
+import pytest
+
+from kombu import Connection, Exchange, Queue, messaging
 
 boto3 = pytest.importorskip('boto3')
 
-from kombu.transport import SQS                 # noqa
-from botocore.exceptions import ClientError     # noqa
+from botocore.exceptions import ClientError  # noqa
+
+from kombu.transport import SQS  # noqa
 
 SQS_Channel_sqs = SQS.Channel.sqs
 
@@ -30,12 +30,19 @@ example_predefined_queues = {
         'url': 'https://sqs.us-east-1.amazonaws.com/xxx/queue-1',
         'access_key_id': 'a',
         'secret_access_key': 'b',
+        'backoff_tasks': ['svc.tasks.tasks.task1'],
+        'backoff_policy': {1: 10, 2: 20, 3: 40, 4: 80, 5: 320, 6: 640}
     },
     'queue-2': {
         'url': 'https://sqs.us-east-1.amazonaws.com/xxx/queue-2',
         'access_key_id': 'c',
         'secret_access_key': 'd',
     },
+    'queue-3.fifo': {
+        'url': 'https://sqs.us-east-1.amazonaws.com/xxx/queue-3.fifo',
+        'access_key_id': 'e',
+        'secret_access_key': 'f',
+    }
 }
 
 
@@ -60,7 +67,7 @@ class QueueMock:
         self.messages = []
 
     def __repr__(self):
-        return 'QueueMock: {} {} messages'.format(self.url, len(self.messages))
+        return f'QueueMock: {self.url} {len(self.messages)} messages'
 
 
 class SQSClientMock:
@@ -149,6 +156,7 @@ class test_Channel:
         predefined_queues_sqs_conn_mocks = {
             'queue-1': SQSClientMock(QueueName='queue-1'),
             'queue-2': SQSClientMock(QueueName='queue-2'),
+            'queue-3.fifo': SQSClientMock(QueueName='queue-3.fifo')
         }
 
         def mock_sqs():
@@ -328,6 +336,14 @@ class test_Channel:
         with pytest.raises(Empty):
             self.channel._get_bulk(self.queue_name)
 
+    def test_is_base64_encoded(self):
+        raw = b'{"id": "4cc7438e-afd4-4f8f-a2f3-f46567e7ca77","task": "celery.task.PingTask",' \
+              b'"args": [],"kwargs": {},"retries": 0,"eta": "2009-11-17T12:30:56.527191"}'     # noqa
+        b64_enc = base64.b64encode(raw)
+        assert self.channel._Channel__b64_encoded(b64_enc)
+        assert not self.channel._Channel__b64_encoded(raw)
+        assert not self.channel._Channel__b64_encoded(b"test123")
+
     def test_messages_to_python(self):
         from kombu.asynchronous.aws.sqs.message import Message
 
@@ -597,6 +613,24 @@ class test_Channel:
         basic_reject_mock.assert_called_with(2)
         assert not basic_ack_mock.called
 
+    def test_reject_when_no_predefined_queues(self):
+        connection = Connection(transport=SQS.Transport, transport_options={})
+        channel = connection.channel()
+
+        mock_apply_backoff_policy = Mock()
+        channel.qos.apply_backoff_policy = mock_apply_backoff_policy
+        queue_name = "queue-1"
+
+        exchange = Exchange('test_SQS', type='direct')
+        queue = Queue(queue_name, exchange, queue_name)
+        queue(channel).declare()
+
+        message_mock = Mock()
+        message_mock.delivery_info = {'routing_key': queue_name}
+        channel.qos._delivered['test_message_id'] = message_mock
+        channel.qos.reject('test_message_id')
+        mock_apply_backoff_policy.assert_not_called()
+
     def test_predefined_queues_primes_queue_cache(self):
         connection = Connection(transport=SQS.Transport, transport_options={
             'predefined_queues': example_predefined_queues,
@@ -648,3 +682,200 @@ class test_Channel:
         channel.connection._deliver.assert_called()
 
         assert len(channel.sqs(queue_name)._queues[queue_name].messages) == 0
+
+    def test_predefined_queues_backoff_policy(self):
+        connection = Connection(transport=SQS.Transport, transport_options={
+            'predefined_queues': example_predefined_queues,
+        })
+        channel = connection.channel()
+
+        def apply_backoff_policy(
+                queue_name, delivery_tag, retry_policy, backoff_tasks):
+            return None
+
+        mock_apply_policy = Mock(side_effect=apply_backoff_policy)
+        channel.qos.apply_backoff_policy = mock_apply_policy
+        queue_name = "queue-1"
+
+        exchange = Exchange('test_SQS', type='direct')
+        queue = Queue(queue_name, exchange, queue_name)
+        queue(channel).declare()
+
+        message_mock = Mock()
+        message_mock.delivery_info = {'routing_key': queue_name}
+        channel.qos._delivered['test_message_id'] = message_mock
+        channel.qos.reject('test_message_id')
+        mock_apply_policy.assert_called_once_with(
+            'queue-1', 'test_message_id',
+            {1: 10, 2: 20, 3: 40, 4: 80, 5: 320, 6: 640},
+            ['svc.tasks.tasks.task1']
+        )
+
+    def test_predefined_queues_change_visibility_timeout(self):
+        connection = Connection(transport=SQS.Transport, transport_options={
+            'predefined_queues': example_predefined_queues,
+        })
+        channel = connection.channel()
+
+        def extract_task_name_and_number_of_retries(delivery_tag):
+            return 'svc.tasks.tasks.task1', 2
+
+        mock_extract_task_name_and_number_of_retries = Mock(
+            side_effect=extract_task_name_and_number_of_retries)
+        channel.qos.extract_task_name_and_number_of_retries = \
+            mock_extract_task_name_and_number_of_retries
+
+        queue_name = "queue-1"
+
+        exchange = Exchange('test_SQS', type='direct')
+        queue = Queue(queue_name, exchange, queue_name)
+        queue(channel).declare()
+
+        message_mock = Mock()
+        message_mock.delivery_info = {'routing_key': queue_name}
+        channel.qos._delivered['test_message_id'] = message_mock
+
+        channel.sqs = Mock()
+        sqs_queue_mock = Mock()
+        channel.sqs.return_value = sqs_queue_mock
+        channel.qos.reject('test_message_id')
+
+        sqs_queue_mock.change_message_visibility.assert_called_once_with(
+            QueueUrl='https://sqs.us-east-1.amazonaws.com/xxx/queue-1',
+            ReceiptHandle='test_message_id', VisibilityTimeout=20)
+
+    def test_predefined_queues_put_to_fifo_queue(self):
+        connection = Connection(transport=SQS.Transport, transport_options={
+            'predefined_queues': example_predefined_queues,
+        })
+        channel = connection.channel()
+
+        queue_name = 'queue-3.fifo'
+
+        exchange = Exchange('test_SQS', type='direct')
+        p = messaging.Producer(channel, exchange, routing_key=queue_name)
+
+        queue = Queue(queue_name, exchange, queue_name)
+        queue(channel).declare()
+
+        channel.sqs = Mock()
+        sqs_queue_mock = Mock()
+        channel.sqs.return_value = sqs_queue_mock
+        p.publish('message')
+
+        sqs_queue_mock.send_message.assert_called_once()
+        assert 'MessageGroupId' in sqs_queue_mock.send_message.call_args[1]
+        assert 'MessageDeduplicationId' in \
+            sqs_queue_mock.send_message.call_args[1]
+
+    @pytest.mark.parametrize('predefined_queues', (
+        {
+            'invalid-fifo-queue-name': {
+                'url': 'https://sqs.us-east-1.amazonaws.com/xxx/queue.fifo',
+                'access_key_id': 'a',
+                'secret_access_key': 'b'
+            }
+        },
+        {
+            'standard-queue.fifo': {
+                'url': 'https://sqs.us-east-1.amazonaws.com/xxx/queue',
+                'access_key_id': 'a',
+                'secret_access_key': 'b'
+            }
+        }
+    ))
+    def test_predefined_queues_invalid_configuration(self, predefined_queues):
+        connection = Connection(transport=SQS.Transport, transport_options={
+            'predefined_queues': predefined_queues,
+        })
+        with pytest.raises(SQS.InvalidQueueException):
+            connection.channel()
+
+    def test_sts_new_session(self):
+        # Arrange
+        connection = Connection(transport=SQS.Transport, transport_options={
+            'predefined_queues': example_predefined_queues,
+            'sts_role_arn': 'test::arn'
+        })
+        channel = connection.channel()
+        sqs = SQS_Channel_sqs.__get__(channel, SQS.Channel)
+        queue_name = 'queue-1'
+
+        mock_generate_sts_session_token = Mock()
+        mock_new_sqs_client = Mock()
+        channel.new_sqs_client = mock_new_sqs_client
+        mock_generate_sts_session_token.side_effect = [
+            {
+                'Expiration': 123,
+                'SessionToken': 123,
+                'AccessKeyId': 123,
+                'SecretAccessKey': 123
+            }
+        ]
+        channel.generate_sts_session_token = mock_generate_sts_session_token
+
+        # Act
+        sqs(queue=queue_name)
+
+        # Assert
+        mock_generate_sts_session_token.assert_called_once()
+
+    def test_sts_session_expired(self):
+        # Arrange
+        connection = Connection(transport=SQS.Transport, transport_options={
+            'predefined_queues': example_predefined_queues,
+            'sts_role_arn': 'test::arn'
+        })
+        channel = connection.channel()
+        sqs = SQS_Channel_sqs.__get__(channel, SQS.Channel)
+        channel.sts_expiration = datetime.utcnow() - timedelta(days=1)
+        queue_name = 'queue-1'
+
+        mock_generate_sts_session_token = Mock()
+        mock_new_sqs_client = Mock()
+        channel.new_sqs_client = mock_new_sqs_client
+        mock_generate_sts_session_token.side_effect = [
+            {
+                'Expiration': 123,
+                'SessionToken': 123,
+                'AccessKeyId': 123,
+                'SecretAccessKey': 123
+            }
+        ]
+        channel.generate_sts_session_token = mock_generate_sts_session_token
+
+        # Act
+        sqs(queue=queue_name)
+
+        # Assert
+        mock_generate_sts_session_token.assert_called_once()
+
+    def test_sts_session_not_expired(self):
+        # Arrange
+        connection = Connection(transport=SQS.Transport, transport_options={
+            'predefined_queues': example_predefined_queues,
+            'sts_role_arn': 'test::arn'
+        })
+        channel = connection.channel()
+        channel.sts_expiration = datetime.utcnow() + timedelta(days=1)
+        queue_name = 'queue-1'
+
+        mock_generate_sts_session_token = Mock()
+        mock_new_sqs_client = Mock()
+        channel.new_sqs_client = mock_new_sqs_client
+        channel._predefined_queue_clients = {queue_name: 'mock_client'}
+        mock_generate_sts_session_token.side_effect = [
+            {
+                'Expiration': 123,
+                'SessionToken': 123,
+                'AccessKeyId': 123,
+                'SecretAccessKey': 123
+            }
+        ]
+        channel.generate_sts_session_token = mock_generate_sts_session_token
+
+        # Act
+        channel.sqs(queue=queue_name)
+
+        # Assert
+        mock_generate_sts_session_token.assert_not_called()
