@@ -1,12 +1,12 @@
 import tempfile
 
-from case import skip
-from case.skip import SkipTest
+import pytest
 
-from kombu import Connection, Exchange, Queue, Consumer, Producer
+import t.skip
+from kombu import Connection, Consumer, Exchange, Producer, Queue
 
 
-@skip.if_win32()
+@t.skip.if_win32
 class test_FilesystemTransport:
 
     def setup(self):
@@ -15,7 +15,7 @@ class test_FilesystemTransport:
             data_folder_in = tempfile.mkdtemp()
             data_folder_out = tempfile.mkdtemp()
         except Exception:
-            raise SkipTest('filesystem transport: cannot create tempfiles')
+            pytest.skip('filesystem transport: cannot create tempfiles')
         self.c = Connection(transport='filesystem',
                             transport_options={
                                 'data_folder_in': data_folder_in,
